#!/usr/local/bin/ruby -Ke
# Thanks:: >>63
# $Id: new_2ch_bbsmenu.rb,v 1.2 2002/01/22 06:03:21 haruyama Exp $
# このファイルはGPLで配付されています.
# このプログラムは無保証です.

require 'connect2ch'
include GetBoardInfo2ch

host,port,path = parse_http_url(ARGV[0])
if host
  body=get_board_info(host,port,path)
  if body
    info=parse_board_info_html(body)
  else
    print "htmlファイルのGETに失敗\n"
  end
else
  info=parse_board_info_html(ARGF)
end
info.each{ |e|
  print Kconv::kconv(e,Kconv::SJIS,Kconv::EUC)
}
